import mitt from 'mitt';
import mapPromise from 'callbag-map-promise';
import filter from 'callbag-filter';
import share from 'callbag-share';

var isFn = function isFn(val) {
  return typeof val === "function";
};

var getPayload = function getPayload(d) {
  return d && d.payload !== undefined ? d.payload : d;
};

var INIT_TYPE = "@REDUX_CALLBAG_INIT@";
var index = (function () {
  for (var _len = arguments.length, epicses = new Array(_len), _key = 0; _key < _len; _key++) {
    epicses[_key] = arguments[_key];
  }

  return function (store) {
    var emitter = mitt();
    var actions = share(function (start, sink) {
      if (start !== 0) return;

      var handler = function handler(ev) {
        return sink(1, ev);
      };

      sink(0, function (t, d) {
        if (t === 2) {
          emitter.off("action", handler);
        }
      });
      emitter.on("action", handler);
    });

    actions.select = function () {
      for (var _len2 = arguments.length, types = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        types[_key2] = arguments[_key2];
      }

      return function (source) {
        if (types.length == 0) types = [INIT_TYPE];
        return filter(function (_ref) {
          var type = _ref.type;
          return types.some(function (_type) {
            if (_type == "INIT") _type = INIT_TYPE;
            return _type == type;
          });
        })(source);
      };
    };

    actions.mapPromise = mapPromise;

    actions.mapSuccessTo = function (actionType, fn) {
      return function (source) {
        source(0, function (t, d) {

          if (t === 1) {
            store.dispatch({
              type: actionType,
              payload: isFn(fn) ? fn(getPayload(d)) : getPayload(d)
            });
          }
        });
        return function (t, sink) {
          if (t !== 0) return;
          source(0, function (t, d) {
            sink(t, d);
          });
        };
      };
    };

    actions.mapFailTo = function (actionType, fn) {
      return function (source) {
        source(0, function (t, d) {

          if (t === 2 && d !== undefined) {
            store.dispatch({
              type: actionType,
              payload: isFn(fn) ? fn(getPayload(d)) : getPayload(d)
            });
          }
        });
        return function (t, sink) {
          if (t !== 0) return;
          source(0, function (t, d) {
            sink(t, d);
          });
        };
      };
    };

    epicses.forEach(function (epics) {
      if (typeof epics === "function") {
        epics(actions, store);
      }
    });
    emitter.emit("action", {
      type: INIT_TYPE
    });
    return function (next) {
      return function (action) {
        emitter.emit("action", action);
        return next(action);
      };
    };
  };
});

export default index;
export { INIT_TYPE };
